from typing import Dict
from .base import BaseAIModel
from .gpt_api import GPTAPI
from .claude_api import ClaudeAPI
from .gemini_api import GeminiAPI
from .exceptions import InvalidProviderError

class Sunsuku:
    
    def __init__(self):
        self.models: Dict[str, BaseAIModel] = {}
    

    def add_model(self, provider: str, api_key: str, model_name: str, **kwargs):
        if provider == 'openai':
            self.models[provider] = GPTAPI(api_key, model_name, **kwargs)
        elif provider == 'anthropic':
            self.models[provider] = ClaudeAPI(api_key, model_name, **kwargs)
        elif provider == 'google':
            self.models[provider] = GeminiAPI(api_key, model_name, **kwargs)
        else:
            raise InvalidProviderError(f'Unsupported provider: {provider}')
    

    def generate_text(self, provider: str, prompt: str) -> str:
        if provider not in self.models:
            raise InvalidProviderError(f'Provider: {provider} not initialized.')
        return self.models[provider].generate_text(prompt)

    def set_model(self, provider: str, model_name: str):
        if provider not in self.models:
            raise InvalidProviderError(f'Provider')

    def clear_message_history(self, provider: str):
        if provider not in self.models:
            raise InvalidProviderError(f"Provider: {provider} not initialized")
        self.models[provider].clear_message_history()

    def get_allowed_models(self, provider: str) -> list:
        if provider not in self.models:
            raise InvalidProviderError(f'Provider {provider} not initialized')
        return self.models[provider].ALLOWED_MODELS
    
    def configure_mode(self, provider: str, **kwargs):
        if provider not in self.models:
            raise InvalidProviderError(f'Provider {provider} not initialized')
        self.models['provider'].configure(**kwargs)




from typing import Any, Dict
import google.generativeai as genai
from .base import BaseAIModel

class GeminiAPI(BaseAIModel):

    ALLOWED_MODELS = ['gemini-1.5-pro', 'gemini-1.5-flash']

    def __init__(self, api_key: str, model_name: str, system_prompt: str = '', max_tokens: int=1024, temperature: float=0):
        super().__init__(api_key, model_name, system_prompt, max_tokens=max_tokens, temperature=temperature)

    
    @property
    def client(self):
        genai.configure(api_key=self.api_key)
        if self.system_prompt:
            return genai.GenerativeModel(model_name=self.model_name, system_instruction=self.system_prompt)
        else:
            return genai.GenerativeModel(model_name=self.model_name)
    

    def generate_text(self, prompt: str) -> str:
        generation_config = genai.types.GenerationConfig(
            max_output_tokens=self.max_tokens,
            temperature=self.temperature
        )
        self.add_message('user', prompt)
        completion = self.client.generate_content(
            contents=self.message_history,
            generation_config=generation_config
        )
        assistant_message = completion.candidates[0].content.parts[0].text
        self.add_message(completion.candidates[0].content.role, assistant_message)
        return assistant_message
    

    def _format_message(self, role: str, content: str) -> Dict[str, Any]:
        return {'role': role, 'parts': content}



from typing import Any, Dict
import openai
from .base import BaseAIModel

class GPTAPI(BaseAIModel):

    ALLOWED_MODELS = ['gpt-4o', 'gpt-4o-mini']

    @property
    def client(self):
        return openai.OpenAI(api_key=self.api_key)
    
    def generate_text(self, prompt: str) -> str:
        self.add_message('user', prompt)
        completion = self.client.chat.completions.create(
            model = self.model_name,
            messages=self.message_history
        )
        assistant_message = completion.choices[0].message.content
        self.add_message(completion.choices[0].message.role, assistant_message)
        return assistant_message

    def _format_message(self, role: str, content: str) -> Dict[str, Any]:
        return {'role': role, 'content': content}




from typing import Any, Dict
import anthropic
from .base import BaseAIModel

class ClaudeAPI(BaseAIModel):

    ALLOWED_MODELS = ["claude-3-5-sonnet-20240620", 'claude-3-haiku-20240307']


    @property
    def client(self):
        return anthropic.Anthropic(api_key=self.api_key)
    
    def generate_text(self, prompt: str) -> str:
        self.add_message('user', prompt)
        completion = self.client.messages.create(
            model=self.model_name,
            messages=self.message_history,
            max_tokens=self.max_tokens
        )
        assistant_message = completion.content[0].text
        self.add_message(completion.role, assistant_message)
        return assistant_message
    

    def _format_message(self, role: str, content: str) -> Dict[str, Any]:
        return {'role': role, 'content': content}



from abc import ABC, abstractmethod
from typing import List, Dict, Any


class BaseAIModel(ABC):

    def __init__(self, api_key: str, model_name: str, system_prompt: str='', max_tokens=1024, **kwargs):
        self.api_key = api_key
        self.system_prompt = system_prompt
        self.max_tokens = max_tokens
        self.set_model(model_name)
        self.message_history :List[Dict[str, str]] = []
        if system_prompt:
            self.add_message('system', system_prompt)
        self.configure(**kwargs)

    
    @property
    @abstractmethod
    def client(self):
        pass

    @property
    @abstractmethod
    def ALLOWED_MODELS(self):
        pass

    def set_model(self, model_name: str):
        if model_name not in self.ALLOWED_MODELS:
            raise ValueError(f'Model name must be one of {self.ALLOWED_MODELS}')
        self.model_name = model_name

    
    def add_message(self, role: str, content: str):
        self.message_history.append(self._format_message(role, content))

    def clear_message_history(self):
        self.message_history.clear()

    @abstractmethod
    def generate_text(self, prompt: str) -> str:
        pass

    @abstractmethod
    def _format_message(self, role: str, content: str) -> Dict[str, Any]:
        pass

    def configure(self, **kwargs):
        for key, value in kwargs.items():
            setattr(self, key, value)
